"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } }

var nonWhitespace = /\S/;
var newlines = /\r\n?|\n/;
var whitespaceTimmer = /^\s+|\s+$/g;

// Trims the whitespace off the lines.
function lineFilter(lines, line) {
  line = line.replace(whitespaceTimmer, "");

  if (line) {
    lines.push(line);
  }

  return lines;
}

// Cleans the whitespace from a text node.
function cleanText(_ref) {
  var value = _ref.value;

  if (nonWhitespace.test(value)) {
    var lines = value.split(newlines);

    lines = lines.reduce(lineFilter, []);

    return lines.join(" ");
  }
}

exports["default"] = function (t) {

  // Helper to create a function call in AST.
  function toFunctionCall(functionName, args) {
    return t.callExpression(t.identifier(functionName), args);
  }

  // Helper to create a function call statement in AST.
  function toFunctionCallStatement(functionName, args) {
    return t.expressionStatement(toFunctionCall(functionName, args));
  }

  // Helper to transform a JSX identifier into a normal reference.
  function toReference(node, identifier) {
    if (t.isJSXIdentifier(node)) {
      return identifier ? t.identifier(node.name) : t.literal(node.name);
    }
    return t.memberExpression(toReference(node.object, true), toReference(node.property, true));
  }

  // Filters out empty children, and transform JSX expressions
  // into normal expressions.
  function childFilter(children, child) {
    if (t.isJSXExpressionContainer(child)) {
      child = child.expression;
    }

    if (t.isJSXEmptyExpression(child)) {
      return children;
    } else if (t.isLiteral(child) && typeof child.value === "string") {
      var text = cleanText(child);
      if (!text) {
        return children;
      }

      child = toFunctionCallStatement("text", [t.literal(text)]);
    }

    children.push(child);
    return children;
  }

  // Helper to transform a call expression into an expression statement.
  function callToStatement(expression) {
    if (t.isCallExpression(expression)) {
      return t.expressionStatement(expression);
    }
    return expression;
  }

  // Helper to flatten out sequence expressions into a top level
  // expression statements.
  function sequenceReducer(nodes, node) {
    if (t.isSequenceExpression(node)) {
      var expressions = node.expressions.map(callToStatement);
      nodes.push.apply(nodes, _toConsumableArray(expressions));
    } else {
      nodes.push(callToStatement(node));
    }
    return nodes;
  }

  return {
    buildChildren: function buildChildren(children) {
      return children.reduce(childFilter, []);
    },

    // Extracts attributes into the appropriate
    // attribute array. Static attributes and the key
    // are placed into static attributes, and expressions
    // are placed into the variadic attributes.
    extractOpenArguments: function extractOpenArguments(attributes) {
      var key = null;
      var statics = [];
      var attrs = [];
      var hasSpread = false;

      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = attributes[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var attribute = _step.value;

          if (t.isJSXSpreadAttribute(attribute)) {
            hasSpread = true;
            attrs.push(attribute);
          } else {
            var _name = attribute.name.name;
            var attr = t.literal(_name);
            var value = attribute.value;

            if (t.isLiteral(value)) {
              statics.push(attr, value);
            } else if (value) {
              value = value.expression;

              if (_name === "key") {
                statics.push(attr, value);
              } else {
                attrs.push([attr, value]);
              }
            } else {
              statics.push(attr, t.literal(true));
            }

            if (_name === "key") {
              key = value;
            }
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator["return"]) {
            _iterator["return"]();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      if (!statics.length) {
        statics = null;
      }
      if (!attrs.length) {
        attrs = null;
      }

      return { key: key, statics: statics, attrs: attrs, hasSpread: hasSpread };
    },

    flattenExpressions: function flattenExpressions(expressions) {
      return expressions.reduce(sequenceReducer, []);
    },

    // Transforms an attribute array into sequential attr calls.
    attrsToAttrCalls: function attrsToAttrCalls(scope) {
      return function (attr) {
        if (t.isJSXSpreadAttribute(attr)) {
          var iterator = scope.generateUidIdentifier("attr");

          return t.forInStatement(t.variableDeclaration("var", [iterator]), attr.argument, toFunctionCallStatement("attr", [iterator, t.memberExpression(attr.argument, iterator, true)]));
        } else {
          return toFunctionCall("attr", attr);
        }
      };
    },

    toFunctionCall: toFunctionCall,

    toFunctionCallStatement: toFunctionCallStatement,

    toReference: toReference
  };
};

;
module.exports = exports["default"];